class Site:
    DRAFTKINGS = 'DRAFTKINGS'
    FANDUEL = 'FANDUEL'
    YAHOO = 'YAHOO'
    FANTASY_DRAFT = 'FANTASY_DRAFT'
    FANBALL = 'FANBALL'
    DRAFTKINGS_CAPTAIN_MODE = 'DRAFTKINGS_CAPTAIN_MODE'


class Sport:
    BASKETBALL = 'BASKETBALL'
    FOOTBALL = 'FOOTBALL'
    HOCKEY = 'HOCKEY'
    BASEBALL = 'BASEBALL'
    GOLF = 'GOLF'
    SOCCER = 'SOCCER'
    CANADIAN_FOOTBALL = 'CANADIAN_FOOTBALL'
    LEAGUE_OF_LEGENDS = 'LEAGUE_OF_LEGENDS'
    WNBA = 'WNBA'
    MMA = 'MMA'
